package main

import (
	"bytes"
	"encoding/csv"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/sqweek/dialog"
	"log"
	"net/http"
	"os"
	"strconv"
	"strings"
	"time"
)

type LootRecord struct {
	Name      string    `json:"player"`
	ItemID    int       `json:"item_id"`
	ItemName  string    `json:"item_name"`
	RollType  string    `json:"roll_type"`
	Timestamp time.Time `json:"timestamp"`
}

func loot() {
	csvPath, err := dialog.File().Title("Select the csv file").Load()
	if err != nil {
		if errors.Is(err, dialog.ErrCancelled) {
			log.Fatalf("Cancelled dialog box, exiting")
		} else {
			log.Fatal(err)
		}
	}

	csvFile, err := os.Open(csvPath)
	if err != nil {
		log.Fatal(err)
	}
	defer csvFile.Close()

	csvReader := csv.NewReader(csvFile)
	records, err := csvReader.ReadAll()
	if err != nil {
		log.Fatalf("Failed to parse csv: %v\n", err)
	}

	loots := make([]LootRecord, 0)
	for i := 1; i < len(records); i++ {
		if strings.Contains(records[i][7], "Personal Loot") {
			continue
		}

		var l = LootRecord{
			Name:     strings.Split(records[i][0], "-")[0],
			ItemName: strings.Trim(records[i][4], "[]"),
		}
		l.ItemID, err = strconv.Atoi(records[i][5])
		if err != nil {
			log.Fatal(err)
		}
		l.Timestamp, err = time.Parse("2006/1/2 15:04:05", fmt.Sprintf("%s %s", records[i][1], records[i][2]))
		if err != nil {
			log.Fatal(err)
		}

		switch records[i][7] {
		case "Mainspec/Need":
			l.RollType = "mainspec"
		case "Offspec":
			l.RollType = "offspec"
		case "Minor Upgrade":
			l.RollType = "minor"
		case "Transmog":
			l.RollType = "transmog"
		default:
			l.RollType = "unknown"
		}

		loots = append(loots, l)
	}

	for _, l := range loots {
		err = uploadLoot(l, "https://forcek.in/loot")
		//err = uploadLootTest(l)
		if err != nil {
			log.Fatal(err)
		}
	}
}

func uploadLoot(l LootRecord, endpoint string) error {
	marshalled, err := json.Marshal(l)
	if err != nil {
		return err
	}
	fmt.Println(string(marshalled))

	req, err := http.NewRequest("POST", endpoint, bytes.NewBuffer(marshalled))
	if err != nil {
		return err
	}

	req.Header.Set("Content-Type", "application/json")
	req.Header.Set("X-API-KEY", config.Apikey)

	resp, err := http.DefaultClient.Do(req)
	if err != nil {
		return err
	}

	fmt.Println(resp.Status)
	return nil
}

func uploadLootTest(l LootRecord) error {
	return uploadLoot(l, "http://localhost:3000/loot")
}
